# - Find SOCI
# Find the SOCI headers and libraries.
#
#  SOCI_INCLUDE_DIRS  - where to find soci/soci.h and backends headers.
#  SOCI_LIBRARIES     - list of libraries when using SOCI.
#  SOCI_FOUND         - true if SOCI core is found.
#  SOCI_SQLITE3_FOUND - true if SOCI SQLite3 backend is found.
#  SOCI_ODBC_FOUND    - true if SOCI ODBC backend is found.

# Look for the main header file.
FIND_PATH(SOCI_INCLUDE_CORE_DIR soci.h)

# Look for the SQLite3 backend header.
FIND_PATH(SOCI_INCLUDE_SQLITE3_DIR soci-sqlite3.h PATHS ${SOCI_INCLUDE_CORE_DIR}/../backends/sqlite3)

# Look for the ODBC backend header.
FIND_PATH(SOCI_INCLUDE_ODBC_DIR soci-odbc.h PATHS ${SOCI_INCLUDE_CORE_DIR}/../backends/odbc)

# Combine found dirs into single list
IF(SOCI_INCLUDE_CORE_DIR)
  SET(SOCI_INCLUDE_DIRS_TMP ${SOCI_INCLUDE_DIRS_TMP} ${SOCI_INCLUDE_CORE_DIR})
ENDIF(SOCI_INCLUDE_CORE_DIR)

IF(SOCI_INCLUDE_SQLITE3_DIR)
  FIND_PATH(SQLITE3_INCLUDE_DIR sqlite3.h)
  IF(NOT SQLITE3_INCLUDE_DIR)
    MESSAGE(STATUS "SOCI SQLite3 backend header was found, but SQLite3 header "
                   "itself is missing, disabling SOCI SQLite3 backend.")
    SET(SOCI_INCLUDE_SQLITE3_DIR NOTFOUND)
  ELSE(NOT SQLITE3_INCLUDE_DIR)
    SET(SOCI_INCLUDE_DIRS_TMP ${SOCI_INCLUDE_DIRS_TMP}
        ${SOCI_INCLUDE_SQLITE3_DIR} ${SQLITE3_INCLUDE_DIR})
  ENDIF(NOT SQLITE3_INCLUDE_DIR)
ENDIF(SOCI_INCLUDE_SQLITE3_DIR)

IF(SOCI_INCLUDE_ODBC_DIR)
  SET(SOCI_INCLUDE_DIRS_TMP ${SOCI_INCLUDE_DIRS_TMP} ${SOCI_INCLUDE_ODBC_DIR})
ENDIF (SOCI_INCLUDE_ODBC_DIR)


# Look for the SOCI core library.
IF(SOCI_INCLUDE_CORE_DIR)
  FIND_LIBRARY(SOCI_CORE_LIBRARY soci_core)
ENDIF(SOCI_INCLUDE_CORE_DIR)

IF (SOCI_CORE_LIBRARY)
  GET_FILENAME_COMPONENT(SOCI_LIB_PATH ${SOCI_CORE_LIBRARY} PATH)
ENDIF (SOCI_CORE_LIBRARY)

# Look for the SOCI SQLite3 library.
IF(SOCI_INCLUDE_SQLITE3_DIR)
  FIND_LIBRARY(SOCI_SQLITE3_LIBRARY soci_sqlite3 PATHS ${SOCI_LIB_PATH})
ENDIF(SOCI_INCLUDE_SQLITE3_DIR)

# Look for the SOCI ODBC library.
IF(SOCI_ODBC_INCLUDE_DIR)
  FIND_LIBRARY(SOCI_ODBC_LIBRARY soci_odbc PATHS ${SOCI_LIB_PATH})
ENDIF(SOCI_ODBC_INCLUDE_DIR)


# Combine found libraries into single list
IF(SOCI_CORE_LIBRARY)
  SET(SOCI_LIBRARIES_TMP ${SOCI_LIBRARIES_TMP} ${SOCI_CORE_LIBRARY})
ENDIF (SOCI_CORE_LIBRARY)

IF(SOCI_SQLITE3_LIBRARY)
  FIND_LIBRARY(SQLITE3_LIBRARY sqlite)
  IF (NOT SQLITE3_LIBRARY)
    MESSAGE(STATUS "SOCI SQLite3 backend library was found, but SQLite3 library "
                   "itself is missing, disabling SOCI SQLite3 backend.")
    SET(SOCI_SQLITE3_LIBRARY NOTFOUND)
  ELSE(NOT SQLITE3_LIBRARY)
    SET(SOCI_LIBRARIES_TMP ${SOCI_LIBRARIES_TMP} ${SOCI_SQLITE3_LIBRARY}
        ${SQLITE3_LIBRARY})
  ENDIF(NOT SQLITE3_LIBRARY)
ENDIF (SOCI_SQLITE3_LIBRARY)

IF(SOCI_ODBC_LIBRARY)
  SET(SOCI_LIBRARIES_TMP ${SOCI_LIBRARIES_TMP} ${SOCI_ODBC_LIBRARY})
ENDIF (SOCI_ODBC_LIBRARY)

# Copy the results to the output variables.
IF(SOCI_INCLUDE_CORE_DIR AND SOCI_CORE_LIBRARY)
  SET(SOCI_FOUND 1)
  SET(SOCI_LIBRARIES ${SOCI_LIBRARIES_TMP})
  SET(SOCI_INCLUDE_DIRS ${SOCI_INCLUDE_DIRS_TMP})

  IF (SOCI_INCLUDE_SQLITE3_DIR AND SOCI_SQLITE3_LIBRARY)
    SET(SOCI_SQLITE3_FOUND 1)
  ELSE (SOCI_INCLUDE_SQLITE3_DIR AND SOCI_SQLITE3_LIBRARY)
    SET(SOCI_SQLITE3_FOUND 0)
  ENDIF (SOCI_INCLUDE_SQLITE3_DIR AND SOCI_SQLITE3_LIBRARY)

  IF (SOCI_INCLUDE_ODBC_DIR AND SOCI_ODBC_LIBRARY)
    SET(SOCI_ODBC_FOUND 1)
  ELSE (SOCI_INCLUDE_ODBC_DIR AND SOCI_ODBC_LIBRARY)
    SET(SOCI_ODBC_FOUND 0)
  ENDIF (SOCI_INCLUDE_ODBC_DIR AND SOCI_ODBC_LIBRARY)
ELSE(SOCI_INCLUDE_CORE_DIR AND SOCI_CORE_LIBRARY)
  SET(SOCI_FOUND 0)
  SET(SOCI_SQLITE3_FOUND 0)
  SET(SOCI_ODBC_FOUND 0)
  SET(SOCI_LIBRARIES)
  SET(SOCI_INCLUDE_DIRS)
ENDIF(SOCI_INCLUDE_CORE_DIR AND SOCI_CORE_LIBRARY)

# Report the results.
IF(NOT SOCI_FOUND)
  SET(SOCI_DIR_MESSAGE
    "SOCI was not found. Make sure SOCI_LIBRARY and SOCI_INCLUDE_DIR are set.")
  IF(NOT SOCI_FIND_QUIETLY)
    MESSAGE(STATUS "${SOCI_DIR_MESSAGE}")
  ELSE(NOT SOCI_FIND_QUIETLY)
    IF(SOCI_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "${SOCI_DIR_MESSAGE}")
    ENDIF(SOCI_FIND_REQUIRED)
  ENDIF(NOT SOCI_FIND_QUIETLY)
ENDIF(NOT SOCI_FOUND)
