/*****************************************************************************
 *
 * libwsw, Copyright (C) 2007 by Alexander Tsvyashchenko
 * contact address: <ejabberd@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "api.h"
#include "api_private.h"
#include "task_indexed.h"

#ifdef USE_SOCI
#include "task_indexed_cached.h"
#endif

#include "utils.h"
#include "drivers/xmpp/driver.h"
//------------------------------------------------------------------------------
using namespace WSW::Core;
using namespace WSW::API_details;
using namespace WSW::Drivers::XMPP;
using namespace WSW::DateTime;
//------------------------------------------------------------------------------
// Use templatized receive functions:
#include "task_indexed_tmpl.h"
//------------------------------------------------------------------------------
typedef WSW::API::ReceiveCollection ReceiveCollection;
//------------------------------------------------------------------------------
// Listing support
//------------------------------------------------------------------------------
namespace
  {
  template <class TaskBase>
  class GetCollectionsTaskBase: public TaskBase
    {
    public:
      GetCollectionsTaskBase(Driver& driver, const Options& opts,
                             const std::string& jid, long first_index, long last_index,
                             const ReceiveCollection& func_coll,
                             const ReceiveStatus& func_status = ReceiveStatus()):
        Task(driver, opts, func_status),
        TaskBase(driver, opts, func_status, first_index, last_index), jid_(jid),
        func_coll_(func_coll) {}
//----//----------------------------------------------------------------------------
      virtual void setupReceiveItemsQuery()
        {
        TaskBase::driver_.retrieveCollections(Filter(jid_), setupRSM_ForItemsQuery(TaskBase::opts_.rsm_collections_limit),
          boost::bind(&TaskBase::template receiveItems<Driver::Collections>, this, _1, _2, _3));
        }
//----//----------------------------------------------------------------------------
      virtual void setupGetItemsNumberQuery()
        {
        RSM_Query rsm;
        rsm.limit = 0;
        TaskBase::driver_.retrieveCollections(Filter(jid_), rsm,
          boost::bind(&TaskBase::template receiveItemsNumber<Driver::Collections>, this, _1, _2, _3));
        }
//----//----------------------------------------------------------------------------
    protected:
      ReceiveCollection func_coll_;
      std::string jid_;
    };
//--//----------------------------------------------------------------------------
  class GetCollectionsDirectTask: public GetCollectionsTaskBase<TaskIndexed>
    {
    public:
      GetCollectionsDirectTask(Driver& driver, const Options& opts,
                               const std::string& jid, long first_index, long last_index,
                               const ReceiveCollection& func_coll,
                               const ReceiveStatus& func_status = ReceiveStatus()):
        Task(driver, opts, func_status),
        GetCollectionsTaskBase<TaskIndexed>(driver, opts, jid, first_index, last_index, func_coll, func_status) {}
//--//----------------------------------------------------------------------------
      virtual void receiveItemsHandleItem(const CollectionInfo& info, unsigned long index)
        {
        func_coll_(std::make_pair(index, info));
        }
//----//----------------------------------------------------------------------------
    };
//--//----------------------------------------------------------------------------
#ifdef USE_SOCI
//--//----------------------------------------------------------------------------
using namespace SOCI;
//--//----------------------------------------------------------------------------
  class GetCollectionsCachedTask: public GetCollectionsTaskBase<TaskIndexedCached>
    {
    public:
      GetCollectionsCachedTask(Driver& driver, Session* session, const Options& opts,
                               const std::string& jid, long first_index, long last_index,
                               const ReceiveCollection& func_coll,
                               const ReceiveStatus& func_status = ReceiveStatus()):
        Task(driver, opts, func_status),
        GetCollectionsTaskBase<TaskIndexedCached>(driver, opts, jid, first_index, last_index, func_coll, func_status),
        rowset_(NULL)
        {
        setSession(session);
        }
//----//----------------------------------------------------------------------------
      ~GetCollectionsCachedTask()
        {
        delete rowset_;
        }
//----//----------------------------------------------------------------------------
      virtual void performGetItemsNumberCacheQuery()
        {
        *session_ << "SELECT colls_number FROM users WHERE with_jid = :jid",
                     into(items_number_, ind_items_number_), use(jid_, "jid");
        }
//----//----------------------------------------------------------------------------
      virtual void postReceiveItemsNumber()
        {
        int val;
        *session_ << "SELECT COUNT(*) FROM users WHERE with_jid = :jid",
                     use(jid_, "jid"), into(val);

        if (!startTransaction(boost::bind(&GetCollectionsCachedTask::postReceiveItemsNumber, this)))
          return;

        if (!val)
          *session_ << "INSERT INTO users(with_jid, colls_number) VALUES(:jid, :colls_number)",
                       use(jid_, "jid"), use(items_number_, "colls_number");
        else
          *session_ << "UPDATE users SET colls_number = :colls_number WHERE with_jid = :jid",
                       use(items_number_, "colls_number"), use(jid_, "jid");
        }
//----//----------------------------------------------------------------------------
      virtual void fetchCachedItems()
        {
        // Work-around for SOCI binding bugs.
        tmp_first_ = first_;
        tmp_last_ = last_;

        rowset_ = new Rowset<CollectionInfoCached>((session_->prepare <<
          "SELECT * FROM collections " <<
          "WHERE with_jid = :jid AND rsm_index >= :first AND rsm_index <= :last " <<
          "ORDER BY rsm_index" << (fl_reversed_?" DESC": ""),
          use(jid_, "jid"), use(tmp_first_, "first"), use(tmp_last_, "last")));

        colls_cached_iter_ = rowset_->begin();
        }
//----//----------------------------------------------------------------------------
      virtual void outputItemFromCache()
        {
        CollectionInfoCached info = *colls_cached_iter_;
        resolveLinks(info);
        outputCollection(info, false);
        }
//----//----------------------------------------------------------------------------
      virtual void receiveItemsHandleItem(const CollectionInfo& in_info, unsigned long index)
        {
        CollectionInfoCached info(in_info);

        // Fills prev_coll_id and next_coll_id.
        unResolveLinks(info);

        info.rsm_index = index;
        info.dirty = false; // We've got it from server - it must be up-to-date and complete ...

        outputCollection(info, true);
        }
//----//----------------------------------------------------------------------------
      void resolveLink(CollectionLink& link, unsigned long id)
        {
        *session_ << "SELECT with_jid, utc FROM collections WHERE coll_id = :id",
                     use(id, "id"), into(link.with), into(link.start);
        }
//----//----------------------------------------------------------------------------
      void resolveLinks(CollectionInfoCached& info)
        {
        if (info.prev_coll_id != CollectionInfoCached::INVALID_ID)
          resolveLink(info.coll.prev, info.prev_coll_id);
        if (info.next_coll_id != CollectionInfoCached::INVALID_ID)
          resolveLink(info.coll.next, info.next_coll_id);
        }
//----//----------------------------------------------------------------------------
      void outputCollection(const CollectionInfoCached& in_info, bool fl_store_in_cache)
        {
        if (fl_store_in_cache)
          {
          CollectionInfoCached info(in_info); // Work-around for SOCI const bugs.
          // Is this collection already in our database?
          eIndicator ind;
          unsigned long coll_id;

          *session_ << "SELECT coll_id FROM collections WHERE with_jid = :jid AND utc = :start",
                       use(info.coll.self.with, "jid"), use(info.coll.self.start, "start"), into(coll_id, ind);

          // We can reach this point only by going through setupReceiveItems; however, it should
          // have been set up transaction for us already.
          startTransaction();

          if (ind != eOK) // No such collection found, insert it from scratch.
            *session_ << "INSERT INTO collections(with_jid, utc, prev_coll_id, next_coll_id, " <<
                         "subject, thread, crypt, extra, rsm_index, dirty) " <<
                         "VALUES(:with_jid, :utc, :prev_coll_id, :next_coll_id, " <<
                         ":subject, :thread, :crypt, :extra, :rsm_index, :dirty)",
                         use(info);
          else // Update existing collection.
            updateCollectionInCache(in_info, coll_id);
          }

        func_coll_(std::make_pair(in_info.rsm_index, in_info.coll));
        }
//----//----------------------------------------------------------------------------
      virtual long isCachePosValid()
        {
        return colls_cached_iter_ != rowset_->end();
        }
//----//----------------------------------------------------------------------------
      virtual CachedItem& cachedItem()
        {
        return *colls_cached_iter_;
        }
//----//----------------------------------------------------------------------------
      virtual bool isDirty()
        {
        return colls_cached_iter_->dirty;
        }
//----//----------------------------------------------------------------------------
      virtual void advanceCachePos()
        {
        ++colls_cached_iter_;
        }
//----//----------------------------------------------------------------------------
      virtual std::string getLockingJID()
        {
        return jid_;
        }
//----//----------------------------------------------------------------------------

    private:
      Rowset<CollectionInfoCached>* rowset_;
      Rowset<CollectionInfoCached>::const_iterator colls_cached_iter_;
      unsigned long tmp_first_, tmp_last_; // Work-around for SOCI binding bugs.
    };
//--//--------------------------------------------------------------------------
#endif // USE_SOCI
//--//--------------------------------------------------------------------------
  }
//------------------------------------------------------------------------------
StatusAPI WSW::API::getCollectionsSync(const std::string& jid, long first_index, long last_index,
                                    const ReceiveCollection& func)
  {
  // We do not allow empty JIDs as it would create too high load on server
  // and is hard to cache efficiently & keep updated anyway.
  if (jid.empty())
    return NO_JID_SPECIFIED;

  TaskIndexed* task =
#ifdef USE_SOCI
    opts_.use_db_cache?
      static_cast<TaskIndexed*>(new GetCollectionsCachedTask(*driver_, session_, opts_, jid, first_index, last_index, func)):
#endif
      static_cast<TaskIndexed*>(new GetCollectionsDirectTask(*driver_, opts_, jid, first_index, last_index, func));

  return executeSync_(task);
  }
//------------------------------------------------------------------------------
void WSW::API::getCollectionsAsync(const std::string& jid, long first_index, long last_index,
                                   const ReceiveCollection& func_coll, const ReceiveStatus& func_status)
  {
  // We do not allow empty JIDs as it would create too high load on server
  // and is hard to cache efficiently & keep updated anyway.
  if (jid.empty())
    {
    func_status(NO_JID_SPECIFIED);
    return;
    }

  TaskIndexed* task =
#ifdef USE_SOCI
    opts_.use_db_cache?
      static_cast<TaskIndexed*>(new GetCollectionsCachedTask(*driver_, session_, opts_, jid, first_index, last_index, func_coll, func_status)):
#endif
      static_cast<TaskIndexed*>(new GetCollectionsDirectTask(*driver_, opts_, jid, first_index, last_index, func_coll, func_status));

  task->init();
  }
//------------------------------------------------------------------------------
