/*****************************************************************************
 *
 * libwsw, Copyright (C) 2007 by Alexander Tsvyashchenko
 * contact address: <ejabberd@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "api.h"
#include "api_private.h"
#include "task_indexed.h"

#ifdef USE_SOCI
#include "task_indexed_cached.h"
#endif

#include "utils.h"
#include "drivers/xmpp/driver.h"
//------------------------------------------------------------------------------
using namespace WSW::Core;
using namespace WSW::API_details;
using namespace WSW::Drivers::XMPP;
using namespace WSW::DateTime;
//------------------------------------------------------------------------------
// Use templatized receive functions:
#include "task_indexed_tmpl.h"
//------------------------------------------------------------------------------
typedef WSW::API::ReceiveMessage ReceiveMessage;
//------------------------------------------------------------------------------
namespace
  {
  template <class TaskBase>
  class GetMessagesTaskBase: public TaskBase
    {
    public:
      GetMessagesTaskBase(Driver& driver, const Options& opts,
                          const CollectionLink& link, long first_index, long last_index,
                          const ReceiveMessage& func_msg,
                          const ReceiveStatus& func_status = ReceiveStatus()):
        Task(driver, opts, func_status),
        TaskBase(driver, opts, func_status, first_index, last_index),
        link_(link), func_msg_(func_msg)
        {}

//----//----------------------------------------------------------------------------
      virtual void setupReceiveItemsQuery()
        {
	// Strange, G++ does not see members of virtual base class without
	// explicit qualification ...
        TaskBase::driver_.retrieveMessages(link_, setupRSM_ForItemsQuery(TaskBase::opts_.rsm_messages_limit),
          boost::bind(&TaskBase::template receiveItems<Driver::Messages>, this, _1, _2, _3),
          boost::bind(&GetMessagesTaskBase<TaskBase>::receiveCollection, this, _1));
        }
//----//----------------------------------------------------------------------------
      virtual void setupGetItemsNumberQuery()
        {
        RSM_Query rsm;
        rsm.limit = 0;
        // We're not interested in collection here, so just pass empty callback.
        TaskBase::driver_.retrieveMessages(link_, rsm,
          boost::bind(&TaskBase::template receiveItemsNumber<Driver::Messages>, this, _1, _2, _3),
          Driver::ReceiveCollection());
        }
//----//----------------------------------------------------------------------------
      // Nothing to be done here - will be overriden for cached version, though.
      virtual void receiveCollection(const CollectionInfo& info) {}
//----//----------------------------------------------------------------------------

    protected:
      ReceiveMessage func_msg_;
      CollectionLink link_;
    };
//--//----------------------------------------------------------------------------
  class GetMessagesDirectTask: public GetMessagesTaskBase<TaskIndexed>
    {
    public:
      GetMessagesDirectTask(Driver& driver, const Options& opts,
                            const CollectionLink& link, long first_index, long last_index,
                            const ReceiveMessage& func_msg,
                            const ReceiveStatus& func_status = ReceiveStatus()):
        Task(driver, opts, func_status),
        GetMessagesTaskBase<TaskIndexed>(driver, opts, link, first_index, last_index, func_msg, func_status)
      {}
//--//----------------------------------------------------------------------------
      virtual void receiveItemsHandleItem(const Message& msg, unsigned long index)
        {
        func_msg_(link_, std::make_pair(index, msg));
        }
//--//----------------------------------------------------------------------------
    };
//--//----------------------------------------------------------------------------
#ifdef USE_SOCI
//--//----------------------------------------------------------------------------
using namespace SOCI;
//--//----------------------------------------------------------------------------
  class GetMessagesCachedTask: public GetMessagesTaskBase<TaskIndexedCached>
    {
    public:
      GetMessagesCachedTask(Driver& driver, Session* session, const Options& opts,
                            const CollectionLink& link, long first_index, long last_index,
                            const ReceiveMessage& func_msg,
                            const ReceiveStatus& func_status = ReceiveStatus()):
        Task(driver, opts, func_status),
        GetMessagesTaskBase<TaskIndexedCached>(driver, opts, link, first_index, last_index, func_msg, func_status),
        fl_update_msgs_number_(false), rowset_(NULL)
      {
      setSession(session);
      }
//--//----------------------------------------------------------------------------
      ~GetMessagesCachedTask()
        {
        delete rowset_;
        }
//--//----------------------------------------------------------------------------
      virtual void performGetItemsNumberCacheQuery()
        {
        int dirty_val;
        eIndicator ind_dirty_val;

        *session_ << "SELECT coll_id, dirty, msgs_number FROM collections " <<
                     "WHERE with_jid = :with AND utc = :start",
                     into(coll_id_, ind_coll_id_),
                     into(dirty_val, ind_dirty_val),
                     into(items_number_, ind_items_number_),
                     use(link_.with, "with"), use(link_.start, "start");

        if (ind_dirty_val != eOK)
          fl_dirty_ = true;
        else
          fl_dirty_ = dirty_val != 0;

        if (fl_dirty_)
          ind_items_number_ = eNoData; // We cannot trust this number anymore ...
        }
//--//----------------------------------------------------------------------------
      virtual void postReceiveItemsNumber()
        {
        fl_update_msgs_number_ = true;
        }
//--//----------------------------------------------------------------------------
      virtual void receiveCollection(const CollectionInfo& in_info)
        {
        // Update it only if it's dirty
        if (fl_dirty_)
          {
          CollectionInfoCached info(in_info);
          unResolveLinks(info);
          info.dirty = false;
          updateCollectionInCache(info, coll_id_);
          }
        }
//----//----------------------------------------------------------------------------
      virtual void fetchCachedItems()
        {
        if (ind_coll_id_ != eOK)
          {
          // No point in trying to fetch messages then, just create
          // appropriate collection and proceed further with query.
          if (!startTransaction(boost::bind(&GetMessagesCachedTask::fetchCachedItems, this)))
            return;
          createCollectionByLink(link_);
          coll_id_ = getCollectionIdByLink(ind_coll_id_, link_);
          assert(ind_coll_id_ == eOK);
          }
        else if (fl_dirty_)
          {
          // We should clear existing messages then as we're going to fetch new ones
          startTransaction();

          *session_ << "DELETE FROM messages WHERE coll_id = :coll_id",
                       use(coll_id_, "coll_id");
          }
        else
          {
          // Work-around for SOCI binding bugs.
          tmp_first_ = first_;
          tmp_last_ = last_;

          rowset_ = new Rowset<MessageCached>((session_->prepare <<
            "SELECT * FROM messages " <<
            "WHERE coll_id = :coll_id AND rsm_index >= :first AND rsm_index <= :last " <<
            "ORDER BY rsm_index" << (fl_reversed_?" DESC": ""),
            use(coll_id_, "coll_id"), use(tmp_first_, "first"), use(tmp_last_, "last")));

          msgs_cached_iter_ = rowset_->begin();
          }
        }
//--//----------------------------------------------------------------------------
      virtual void outputItemFromCache()
        {
        outputMessage(*msgs_cached_iter_, false);
        }
//--//----------------------------------------------------------------------------
      virtual void receiveItemsHandleItem(const Message& msg, unsigned long index)
        {
        MessageCached info(msg);
        info.rsm_index = index;
        outputMessage(info, true);
        }
//--//----------------------------------------------------------------------------
      void outputMessage(const MessageCached& msg, bool fl_store_in_cache)
        {
        if (fl_store_in_cache)
          updateMessageInCache(msg, coll_id_);

        func_msg_(link_, std::make_pair(msg.rsm_index, msg.msg));
        }
//----//--------------------------------------------------------------------------
      virtual void finalize()
        {
        if (fl_update_msgs_number_)
          {
          startTransaction();

          *session_ << "UPDATE collections SET msgs_number = :msgs_number " <<
                       "WHERE coll_id = :coll_id",
                       use(items_number_, "msgs_number"), use(coll_id_, "coll_id");
          }

        GetMessagesTaskBase<TaskIndexedCached>::finalize();
        }
//----//--------------------------------------------------------------------------
      virtual long isCachePosValid()
        {
        return msgs_cached_iter_ != rowset_->end();
        }
//----//----------------------------------------------------------------------------
      virtual CachedItem& cachedItem()
        {
        return *msgs_cached_iter_;
        }
//----//----------------------------------------------------------------------------
      virtual bool isDirty()
        {
        return fl_dirty_;
        }
//----//----------------------------------------------------------------------------
      virtual void advanceCachePos()
        {
        ++msgs_cached_iter_;
        }
//----//--------------------------------------------------------------------------
      virtual std::string getLockingJID()
        {
        return link_.with;
        }
//----//----------------------------------------------------------------------------

    private:
      unsigned long coll_id_;
      eIndicator ind_coll_id_;
      bool fl_dirty_, fl_update_msgs_number_;

      Rowset<MessageCached>* rowset_;
      Rowset<MessageCached>::const_iterator msgs_cached_iter_;
      unsigned long tmp_first_, tmp_last_; // Work-around for SOCI binding bugs.
    };
//--//--------------------------------------------------------------------------
#endif // USE_SOCI
//--//--------------------------------------------------------------------------
  }
//------------------------------------------------------------------------------
StatusAPI WSW::API::getMessagesSync(const CollectionLink& link, long first_index, long last_index,
                                    const ReceiveMessage& func_msg)
  {
  TaskIndexed* task =
#ifdef USE_SOCI
    opts_.use_db_cache?
      static_cast<TaskIndexed*>(new GetMessagesCachedTask(*driver_, session_, opts_, link, first_index, last_index, func_msg)):
#endif
      static_cast<TaskIndexed*>(new GetMessagesDirectTask(*driver_, opts_, link, first_index, last_index, func_msg));

  task->init();

  while (!task->done())
    driver_->idle();

  StatusAPI res = task->result();
  delete task;

  return res;
  }
//------------------------------------------------------------------------------
void WSW::API::getMessagesAsync(const CollectionLink& link, long first_index, long last_index,
                                const ReceiveMessage& func_msg, const ReceiveStatus& func_status)
  {
  TaskIndexed* task =
#ifdef USE_SOCI
    opts_.use_db_cache?
      static_cast<TaskIndexed*>(new GetMessagesCachedTask(*driver_, session_, opts_, link, first_index, last_index, func_msg, func_status)):
#endif
      static_cast<TaskIndexed*>(new GetMessagesDirectTask(*driver_, opts_, link, first_index, last_index, func_msg, func_status));

  task->init();
  }
//------------------------------------------------------------------------------
