/*****************************************************************************
 *
 * libwsw, Copyright (C) 2007 by Alexander Tsvyashchenko
 * contact address: <ejabberd@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef WSW_CORE_H
#define WSW_CORE_H
//------------------------------------------------------------------------------
#include "utils.h"
#include <list>
//------------------------------------------------------------------------------
namespace WSW
  {
  // Data structures for core XEP-136 objects.
  // All text fields are expected to be stored in UTF-8.
  namespace Core
    {
    // Represents XEP-136 message object.
    struct Message
      {
      // We treat notes as special kinds of messages which can be differentiated
      // in user code by special Direction value.
      enum Direction { FROM, TO, NOTE };

      Direction direction;
      std::string body, name;
      // We do not support 'secs' attribute on the level of data structures
      // for uniformity, but the driver will convert this UTC time internally
      // to 'secs' during upload and back to UTC when retrieving messages.
      WSW::DateTime::TimeType utc;

      std::string getDirectionAsString() const;
      void setDirectionFromString(const char* name);
      };

    // Represents information that according to XEP-136 is enough
    // to uniquely identify collection.
    struct CollectionLink
      {
      std::string with;
      WSW::DateTime::TimeType start;

      bool isValid() const;
      bool operator <(const CollectionLink& other_link) const;

      CollectionLink() {}
      CollectionLink(const std::string& with_in, const WSW::DateTime::TimeType& start_in);
      };

    // Represents full information about collection, but without its messages.
    struct CollectionInfo
      {
      // prev and next, if valid, represent links to previous and next collections
      // respectively.
      CollectionLink self, next, prev;
      std::string subject, thread, extra;
      bool crypt;

      CollectionInfo();
      };

    // Represents full information about collection including its messages.
    struct Collection
      {
      typedef std::list<Message> Messages;

      CollectionInfo info;
      Messages messages;
      };

    // Represents RSM subitem in query to the server.
    struct RSM_Query
      {
      enum Mode { NONE, BEFORE, AFTER, INDEX };
      const static unsigned long NO_LIMIT = (unsigned long)-1;

      Mode mode;
      std::string value;
      unsigned long limit;

      std::string getModeAsString() const;

      RSM_Query();
      };

    // Represents RSM subitem in answer from the server.
    struct RSM_Result
      {
      std::string first, last;
      WSW::DateTime::TimeType changed;
      unsigned long count, first_index;
      const static unsigned long NO_VALUE = (unsigned long)-1;

      RSM_Result();
      };

    // Represents filtering expression supported by most XEP-136 archive
    // management commands.
    struct Filter
      {
      std::string with;
      WSW::DateTime::TimeType start, end;

      Filter(const std::string jid = "");
      };

    // Represents single modification element in server results on
    // "modified" request.
    struct Modification
      {
      // Used in replication command to specify what exactly happened to the collection
      // in question.
      enum ModificationType { CHANGED, REMOVED };

      ModificationType type;
      CollectionLink link;
      std::string by;
      };
    };
  };
//------------------------------------------------------------------------------

#endif // WSW_CORE_H
