/*****************************************************************************
 *
 * libwsw, Copyright (C) 2007 by Alexander Tsvyashchenko
 * contact address: <ejabberd@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef WSW_DRIVERS_XMPP_DRIVER_H
#define WSW_DRIVERS_XMPP_DRIVER_H

#include "../../core.h"

//------------------------------------------------------------------------------
namespace WSW
  {
  namespace Drivers
    {
    namespace XMPP
      {
      enum Status
        {
        OK,
        SERVER_ERROR
        };

      extern const char* XMLNS_MESSAGE_ARCHIVING;
      extern const char* XMLNS_RESULT_SET_MANAGEMENT;

      // This interface has to be implemented to provide support for
      // XMPP libraries for libwsw. Here by 'XMPP library' I mean
      // some set of functionality that allows to:
      // 1) Send and receive XMPP stanzas to server in asynchronous way.
      // 2) Compose XMPP stanzas and parse XMPP stanzas received from the
      //    server.
      //
      // Typically, both of these are provided by the same library, but this
      // is not necessary - for example, you could easily combine any network
      // library with any XML parsing library to get the same effect.
      //
      // The main purpose of this interface is to allow seamless integration
      // of libwsw into existing applications without the need to bring there
      // completely new and incompartible code for networking and XML parsing,
      // but rather utilizing existing libraries application uses by providing
      // thin wrapper on it.
      //
      // Please note that implementation MUST be asychronous!
      class Driver
        {
        public:
          typedef std::list<WSW::Core::CollectionInfo> Collections;
          typedef std::list<WSW::Core::Message> Messages;
          typedef std::list<WSW::Core::Modification> Modifications;

          typedef boost::function<void (Status)> ReceiveStatus;
          typedef boost::function<void (const Collections&, const WSW::Core::RSM_Result&, Status)> ReceiveCollections;
          typedef boost::function<void (const WSW::Core::CollectionInfo&)> ReceiveCollection;
          typedef boost::function<void (const Messages&, const WSW::Core::RSM_Result&, Status)> ReceiveMessages;
          typedef boost::function<void (const Modifications&, const WSW::Core::RSM_Result&, Status)> ReceiveModifications;

          virtual void initialize(const ReceiveStatus& func) = 0;
          // This function should send request to the server to fetch collections
          // satisfying filter and rsm restrictions, and later pass received
          // collections to func() callback.
          virtual void retrieveCollections(const WSW::Core::Filter& filter, const WSW::Core::RSM_Query& rsm, const ReceiveCollections& func) = 0;
          // This function should send request to the server to fetch messages
          // belonging to collection link and satisfying rsm restrictions, and
          // later pass received collection (which is fetched as part of
          // messages retrieval) to func_coll() callback and messages -
          // to func_msg() callback.
          virtual void retrieveMessages(const WSW::Core::CollectionLink& link, const WSW::Core::RSM_Query& rsm,
                                        const ReceiveMessages& func_msgs, const ReceiveCollection& func_coll) = 0;
          // This function should send request to the server to perform replication:
          // i.e. return all modified items since some specified time,
          // and later pass received modifications to func() callback.
          // Please note that currently this function should support two modes
          // for replication: XEP-136 standard mode and extension for 'start'
          // attribute usage instead of 'after' RSM element.
          virtual void retrieveModifications(const WSW::Core::Filter& filter, const WSW::Core::RSM_Query& rsm, const ReceiveModifications& func) = 0;
          // This function should send request to the server to upload
          // collection, and later pass upload result to func_status
          // callback.
          virtual void updateCollection(const WSW::Core::Collection& collection, const ReceiveStatus& func_status) = 0;

          // This function should return the full JID representing the
          // user currently associated with this connection to XMPP server.
          virtual std::string getJID() const = 0;
          // This function is called for synchronous mode emulation, so
          // it should perform any networking activity that might be needed
          // to receive results or just yield or sleep if nothing has to be
          // done. Typically, it's a good idea to put here some analog to
          // select() call, which will suspend execution until some activity
          // is detected on connection, if this is supported by the library
          // used.
          virtual void idle() = 0;
        };
      }
    }
  }
//------------------------------------------------------------------------------
#endif // WSW_DRIVERS_XMPP_DRIVER_H
