/*****************************************************************************
 *
 * libwsw, Copyright (C) 2007 by Alexander Tsvyashchenko
 * contact address: <ejabberd@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "api.h"
#include "api_private.h"
#include "task_indexed.h"
#include "drivers/xmpp/driver.h"
//------------------------------------------------------------------------------
using namespace WSW::Core;
using namespace WSW::Math;
using namespace WSW::API_details;
using namespace WSW::Drivers::XMPP;
using namespace WSW::DateTime;
//------------------------------------------------------------------------------
TaskIndexed::TaskIndexed(WSW::Drivers::XMPP::Driver& driver, const WSW::API::Options& opts,
                         const WSW::API::ReceiveStatus& func_status,
                         long first_index, long last_index):
  Task(driver, opts, func_status),
  fl_items_len_defined_(false),
  fl_reversed_(false),
  first_index_(first_index),
  last_index_(last_index)
  {}
//------------------------------------------------------------------------------
bool TaskIndexed::calculateIndices()
  {
  if (fl_items_number_defined_)
    {
    // If we know there are no items - there's no need to proceed further
    if (!items_number_)
      {
      finished();
      return false;
      }

    // Recalculate indices into absolute values.
    if (first_index_ < 0)
      first_ = items_number_ + first_index_;
    else
      first_ = first_index_;

    if (last_index_ < 0)
      last_ = items_number_ + last_index_;
    else
      last_ = last_index_;

    if (first_ > last_)
      {
      std::swap(first_, last_);
      fl_reversed_ = true;
      }
    else
      fl_reversed_ = false;

    if (last_ < 0 || fl_items_number_defined_ && first_ >= (long)items_number_)
      {
      cancel(WSW::API::INVALID_INDICES);
      return false;
      }

    if (first_ < 0)
      first_ = 0;

    if (last_ >= (long)items_number_)
      last_ = items_number_ - 1;
    }
  else
    {
    first_ = first_index_;
    last_ = last_index_;

    // Preserve "-1" position as highest element
    if (last_ != -1 && (first_ > last_ || first_ == -1))
      {
      std::swap(first_, last_);
      fl_reversed_ = true;
      }
    else
      fl_reversed_ = false;
    }

  // This should work OK for both cases
  short sf = sign(first_), sl = sign(last_);
  if (sf == sl || sf == 0 && sl >= 0)
    {
    items_len_ = std::abs(first_ - last_) + 1;
    fl_items_len_defined_ = true;
    }

  first_item_index_ = first_;

  return true;
  }
//----------------------------------------------------------------------------------
RSM_Query TaskIndexed::setupRSM_ForItemsQuery(unsigned long limit)
  {
  RSM_Query rsm;

  if (fl_items_len_defined_)
    rsm.limit = std::min(limit, items_len_ - items_received_);
  else
    rsm.limit = limit;

  if (item_last_.empty())
    {
    long index = first_item_index_;

    assert(fl_items_len_defined_ || first_ == first_item_index_);
    assert(fl_items_len_defined_ || (first_ == 0 || !fl_reversed_) && last_ == -1);

    if (fl_items_number_defined_ &&
        (!fl_reversed_ && index == items_number_ - 1 ||
         fl_reversed_ && index + items_len_ == items_number_) ||
        !fl_items_number_defined_ && fl_reversed_ && last_ == -1)
      rsm.mode = RSM_Query::BEFORE;
    else if (index || fl_reversed_ && items_len_ > rsm.limit)
      {
      assert(index >= 0 && (!fl_reversed_ || fl_items_len_defined_));
      rsm.mode = RSM_Query::INDEX;
      if (fl_reversed_)
        index += items_len_ - rsm.limit;
      rsm.value = boost::lexical_cast<std::string>(index);
      }
    // If we're starting from zero index - no need to use any special mode at all
    }
  else
    {
    rsm.mode = fl_reversed_? RSM_Query::BEFORE: RSM_Query::AFTER;
    rsm.value = item_last_;
    }

  return rsm;
  }
//----------------------------------------------------------------------------------
bool TaskIndexed::verifyRSM(const RSM_Result& rsm, unsigned long received_size)
  {
  // Check that all indices and collections number are as expected
  if (!fl_reversed_ &&
      rsm.first_index != RSM_Result::NO_VALUE &&
      rsm.first_index != first_item_index_ + items_received_ ||

      fl_reversed_ &&
      rsm.first_index != RSM_Result::NO_VALUE &&
      fl_items_len_defined_ &&
      rsm.first_index != first_item_index_ + items_len_ - items_received_ - received_size ||

      fl_items_len_defined_ &&
      items_len_ > rsm.count ||
      
      fl_items_number_defined_ &&
      rsm.count != items_number_)
    {
    cancel(WSW::API::CHANGE_DURING_OPERATION);
    return false;
    }
  return true;
  }
//----------------------------------------------------------------------------------
void TaskIndexed::postReceiveItems(const RSM_Result& rsm, unsigned long received_size)
  {
  items_received_ += received_size;

  if (rsm.count < items_received_ || items_received_ > items_len_)
    {
    cancel(WSW::API::ERROR_SERVER);
    return;
    }

  if (items_received_ == items_len_)
    receiveItemsQueryFinished(); // We've got everything for current query, finish our execution by calling
                                 // next item in chain.
  else
    {
    if (!fl_reversed_)
      item_last_ = rsm.last;
    else
      item_last_ = rsm.first;

    setupReceiveItemsQuery();
    }
  }
//----------------------------------------------------------------------------------
void TaskIndexed::setupGetItemsNumber()
  {
  // We can answer query without knowing items number in two cases:
  // 1) RSM modes can be used to specify starting position
  // 2) We either know number of items to fetch or know that the whole range should be fetched
  //
  // This translates into the following possible intervals:
  // * [N1, N2] where N1 >= 0 and N2 >= -1 (starting position is either 0 or can be specified with "INDEX",
  //                                        number of items is either known or the whole range)
  // * [-1, N] where N <= 0 (starting position is <before/>, number of items is either known or the whole range)
  if (first_index_ >= 0 && last_index_ >= -1 ||
      first_index_ == -1 && last_index_ <= 0 || fl_items_number_defined_)
    fetchItems();
  else
    setupGetItemsNumberQuery();
  }
//----------------------------------------------------------------------------------
void TaskIndexed::init()
  {
  setupGetItemsNumber();
  }
//--//----------------------------------------------------------------------------
void TaskIndexed::fetchItems()
  {
  if (!calculateIndices())
    return;

  items_received_ = 0;

  setupReceiveItemsQuery();
  }
//----------------------------------------------------------------------------------
void TaskIndexed::receiveItemsQueryFinished()
  {
  finished();
  }
//----------------------------------------------------------------------------------
